# Alarm Manager

AlarmManager is a Tango device server that supports the configuration pf [AlarmHandlers](https://gitlab.elettra.eu/cs/ds/alarm-handler). AlarmHandlers are designed to be modular, so that different AlarmManagers can be configured to handle different groups/subsets of AlarmHandlers.
AlarmManager can display the selected alarm configuration as attributes (_tag_, _formula_, _message_, ... see full list below) or can Load/Modify alarms with the configuration provided in the attributes (_tag_, _formula_, _message_, ... see full list below)

## Configuration

Configuration uses both device/class and free properties.

### Device properties

Device properties are for configurations that apply to the entire device.

| Name | Description | Type | Default Value |
|------|-----|:-----:|:-----:|
| HandlerList | List of AlarmHandler devices to be managed | Array of String | none |
| PropertyList | List of free properties each one containing list of AlarmHandler devices to be managed | Array of String | none |

### Device/Class properties

Device properties are for configurations that apply to the entire device. The same properties can also be defined as Class properties, in which case they apply to all devices of the AlarmHandler Class in which the Device Property of the same name is not defined.

| Name | Description | Type | Default Value |
|------|-----|:-----:|:-----:|
| MaxSearchSize | Max size of search result | DevLong[] | 1000 |
| DefaultArchiver | Default HDB++ event subscriber where to store alarm history | String |  |
| DefaultStrategy | Default HDB++ archive strategy for alarms | String |  |


## Interface

### Attributes

#### Static Attributes

| Name | Attr. type | R/W type | Data type | Description |
|------|:-----:|:-----:|:-----:|-----|
| tag | Scalar | READ | DEV_STRING | Alarm _tag_: name of the alarm as just the tag or the fqdn with the name of the AlarmHandler. Once written, if the alarm already exists, _formula_, _message_, _priority_, _group_, _shlvd_time_, _on_delay_, _off_delay_, _on_command_, _off_command_, _receivers_ are filled with the current configuration |
| formula | Scalar | READ | DEV_STRING | Alarm _formula_ |
| message | Scalar | READ | DEV_STRING | Alarm _message_ |
| priority | Scalar | READ | DEV_STRING | Alarm _priority_: **highest**, **high**, **medium**, **low**, **lowest**, **fault**, **alarm**, **log** |
| group | Scalar | READ | DEV_STRING | Alarm _group_ |
| shlvd_time | Scalar | READ | DEV_LONG | Alarm _shlvd_time_ in minutes |
| on_delay | Scalar | READ | DEV_LONG | Alarm _on_delay_ in seconds: delay before alarm becomes active |
| off_delay | Scalar | READ | DEV_LONG | Alarm _off_delay_ in seconds: delay before alarm returns normal |
| on_command | Scalar | READ | DEV_STRING | Alarm _on_command_: command executed whene the alarm becomes active |
| off_command | Scalar | READ | DEV_STRING | Alarm _off_command_: command executed whene the alarm returns normal |
| receivers | Scalar | READ | DEV_STRING | Alarm _receivers_. Recipients of [AlarmNotify](https://gitlab.elettra.eu/cs/ds/alarm-notify) notifications |
| enabled | Scalar | READ | DEV_BOOLEAN | Alarm _enabled_ |
| url | Scalar | READ | DEV_STRING | Alarm _url_ |
| handler | Scalar | READ | DEV_STRING | AlarmHandler device name where the alarm specified in _tag_ is already configured, or where _Load_ command is going to load the alarm |
| archiver | Scalar | READ | DEV_STRING | HDB++ EventSubscriber where the alarm specified in _tag_ is already configured, or where _AddArchiver_ command is going to configure the archiving of this alarm |
| strategy | Scalar | READ | DEV_STRING | HDB++ strategy already configured for the alarm specified in _tag_, or to be configured for this alarm with _AddArchiver_ command |
| default_archiver | Scalar | READ | DEV_BOOLEAN | Use DefaultArchiver property as archiver |
| sync_write | Scalar | READ | DEV_BOOLEAN | If true fill in also the updated attribute write part when _tag_ is written |
| alarmList | Spectrum | READ | DEV_STRING | List of all alarm names |
| alarmFrequency | Spectrum | READ | DEV_DOUBLE | List of alarms frequency, i.e. the number of times alarms have occurred since the last  _ResetStatistics_ command |
| handlerStatus | Spectrum | READ | DEV_STRING | List of all AlarmHandlers and their status. Format described below |

* _handlerStatus_
    ```
    <AlarmHandler device FQDN>: <Tango State>
    Example:
    tango://srv-tango-srf-01.fcs.elettra.trieste.it:20000/alarm/fcs/archiving: RUNNING
    ango://srv-tango-srf-01.fcs.elettra.trieste.it:20000/alarm/fcs/climate: RUNNING
    ```

### Commands

| Name | Input argument | Output argument | Description |
|------|:-----:|:-----:|-----|
| AddArchiver | DEV_VOID | DEV_VOID | Configure archiving of alarm specified in _tag_ attribute on _archiver_ device with _strategy_ |
| AddArchiverConf | DEVVAR_STRINGARRAY | DEV_VOID | Same as _AddArchiver_, but with the following argin: [0] AlarmName, [1] Archiver, [2] Strategy. If just AlarmName, use default archiver and strategy |
| GetAlarmInfo | DEVVAR_STRINGARRAY | DEVVAR_STRINGARRAY | Returns the complete status of the alarm whose name is passed as input argument [0]. The return value is an array of strings _“key=value;”_. Other input arguments may specify keys to be included in the response. Format described below |
| Load | DEV_VOID | DEV_VOID | Load a new alarm on _handler_ AlarmHandler device with the configuration specified in _tag_, _formula_, _message_, _priority_, _group_, _shlvd_time_, _on_delay_, _off_delay_, _on_command_, _off_command_, _receivers_ |
| LoadConf | DEV_STRING | DEV_VOID | Load a new alarm as a string of concatenated _“key=value;”_. Possible keys are the name of the attributes used by the _Load_ command. Note: The _SearchAlarm_ command returns strings that have the correct syntax for the _LoadConf_ and _ModifyConf_ input arguments |
| Modify | DEV_VOID | DEV_VOID | Modify an existing alarm with the configuration specified in _tag_, _formula_, _message_, _priority_, _group_, _shlvd_time_, _on_delay_, _off_delay_, _on_command_, _off_command_, _receivers_ |
| ModifyConf | DEV_STRING | DEV_VOID | Modify an existing alarm. Input argument has the same syntax of the _LoadConf_ command |
| ReLoadAll | DEV_VOID | DEV_VOID | Reaload the configuration of all alarms from attribute properties |
| Remove | DEV_VOID | DEV_VOID | Remove completely an alarm. Its configuration is deleted from the attribute properties |
| RemoveConf | DEV_STRING | DEV_VOID | Remove completely the alarm specified in _tag_. Its configuration is deleted from the attribute properties |
| Rename | DEVVAR_STRINGARRAY | DEV_VOID | Rename an alarm from argin[0] to argin[1] possibly switching to a different AlarmHandler if the device part of the FQDN is different. |
| ResetStatistics | DEV_VOID | DEV_VOID | Reset frequency counters calling _ResetStatistics_ on all AlarmHandlers |
| SearchAlarmConf | DEV_STRING | DEVVAR_STRINGARRAY | Returns the list of configured alarms that match the name of the filter string. If the filter string is empty or *, the command returns the configuration of all alarms. The Alarm configuration is returned as a concatenated _“key=value;”_ string, with the same syntax as the Load command |
| SearchAlarmName | DEV_STRING | DEVVAR_STRINGARRAY | Returns the list of configured alarms that match the name of the filter string. If the filter string is empty or *, the command returns the configuration of all alarms. The Alarm configuration is returned as a concatenated _“key=value;”_ string, with the same syntax as the Load command |

* _GetAlarmInfo_
Returns the complete status of the alarm whose name is passed as the input argument.
The return value is an array of strings_“key=value;”_. The keys are those described in the attribute properties plus:
    * value=one of the alarm state enum labels
    * attr_values=concatenation of “attrname=value;” of the last value of attributes evaluated in the formula
    * quality=the tango quality
    * exception=Reason:.. Desc:.. Origin:..
    * shelved=true/false
    * ack=ACK/NACK
    * audible=true/false
    * on_counter=how many times it evaluated consecutively true
    * on_counter=how many times it evaluated consecutively false
    * freq_counter=how many times is has been evaluated since last ResetStatistics
    * silent_time_remaining=how many minutes till when it will come back from shelved or silenced

	Example:
	```
	[0]	 tag=ks_prealarm_sf6
	[1]	 state=NORM
	[2]	 message="Pre Alarm SF6 on modulator KS"
	[3]	 url=
	[4]	 formula=(ks/interlock/sf6_ks.01/StatSF6[1] == 0)
	[5]	 values={"ks/interlock/sf6_ks.01/statsf6[1]":1}
	[6]	 time=2025-02-09  07:33:19.411087
	[7]	 quality=ATTR_VALID
	[8]	 enabled=1
	[9]	 shelved=0
	[10]	 ack=1
	[11]	 audible=0
	[12]	 counter=2
	[13]	 freq_counter=7
	[14]	 on_delay=0
	[15]	 off_delay=0
	[16]	 priority=high
	[17]	 shlvd_time=-1
	[18]	 shlvd_end=-1
	[19]	 group=gr_ctrl|gr_linac
	[20]	 on_command=
	[21]	 off_command=
	[22]	 receivers=
	```

### AlarmManager GUI

AlarmManager is designed to facilitate the navigation and configuration of alarms on a set of AlarmHandlers, for example, [AlarmManager GUI](https://gitlab.elettra.eu/cs/gui/alarm-manager) displays in a tree the list of alarms read from the _alarmList_ attribute. When an alarm is selected in the tree, _tag_ and _handler_ are written accordingly and all others contain the updated configuration, see:

![screenshot](images/alarm_manager.png)

This can then be modified or loaded as a new alarm if the tag attribute has changed.
